'use strict';

var generateScriptableBanner = require('@scriptables/manifest/generateScriptableBanner');
var normalizeManifest = require('@scriptables/manifest/normalizeManifest');
var validateManifest = require('@scriptables/manifest/validateManifest');
var fs = require('fs');
var promises = require('fs/promises');
var path = require('path');
var logger = require('./logger');
var utils = require('./utils');

const DEFAULT_MANIFEST_EXTENSIONS = [".manifest.json", ".manifest", ".json"];
function scriptableBanner(options = {}) {
  const {
    logLevel = "auto",
    manifestExtensions = DEFAULT_MANIFEST_EXTENSIONS,
    warnOnMissingManifest = true,
    warnOnInvalidManifest = true,
    warnOnMissingEntry = true,
    allowExtraManifestKeys = true,
    getManifestPath,
    patterns = ["*.{ts,tsx,js,jsx}"]
  } = options;
  return {
    name: "scriptable-banner",
    setup(build) {
      const level = logLevel === "auto" ? logger.convertEsbuildLogLevel(build.initialOptions.logLevel || "info") : logLevel;
      const logger$1 = logger.createLogger("scriptable-banner", { level });
      let manifestCache = /* @__PURE__ */ new Map();
      if (!build.initialOptions.metafile) {
        logger$1.warn(
          'Warning: "metafile" option was not enabled. Automatically setting "metafile: true" to allow scriptable plugin to modify output files.'
        );
        build.initialOptions.metafile = true;
      }
      build.onStart(() => {
        manifestCache.clear();
      });
      build.onResolve({ filter: /.*/ }, async (args) => {
        if (args.kind !== "entry-point" || !utils.isMatch(args.path, patterns)) {
          return null;
        }
        const normalizedPath = utils.normalizeAndResolvePath(args.path);
        if (manifestCache.has(normalizedPath)) {
          return null;
        }
        const dir = path.dirname(args.path);
        const baseFileName = path.basename(args.path).replace(/\.[tj]sx?$/, "");
        const manifestPath = getManifestPath ? getManifestPath(args.path) : manifestExtensions.map((ext) => path.resolve(dir, `${baseFileName}${ext}`)).find(fs.existsSync);
        if (manifestPath) {
          try {
            const manifestContent = await promises.readFile(manifestPath, "utf8");
            const manifest = normalizeManifest.normalizeManifest(JSON.parse(manifestContent));
            try {
              validateManifest.validateManifest(manifest, allowExtraManifestKeys);
              manifestCache.set(normalizedPath, manifest);
            } catch (error) {
              if (warnOnInvalidManifest && error instanceof Error) {
                logger$1.warn(`Invalid manifest file: ${manifestPath} - ${error.message}`, error);
              }
            }
          } catch (error) {
            logger$1.warn(`Failed to read manifest file: ${manifestPath}`, error);
          }
        } else if (warnOnMissingManifest) {
          logger$1.warn(`No manifest file found for entry point: ${args.path}`);
        }
        return null;
      });
      build.onEnd(async (result) => {
        logger$1.time("Banner Processing");
        if (!result.metafile || Object.keys(result.metafile.outputs).length === 0) {
          logger$1.warn(
            'Skipping banner addition because "metafile" is not available or contains no outputs. Ensure "metafile: true" is set in esbuild options if write is enabled.'
          );
          return;
        }
        const stats = {
          total: 0,
          success: 0,
          skipped: 0,
          failed: 0
        };
        const outputs = Object.keys(result.metafile.outputs);
        stats.total = outputs.length;
        logger$1.info(`Processing ${stats.total} files...`);
        for (const [index, outputPath] of outputs.entries()) {
          const progress = Math.round((index + 1) / stats.total * 100);
          logger$1.verbose(`[${progress}%] Processing: ${outputPath}`);
          const outputMeta = result.metafile.outputs[outputPath];
          const entryPoint = outputMeta?.entryPoint ?? "";
          const normalizedEntryPath = entryPoint ? utils.normalizeAndResolvePath(entryPoint) : "";
          if (!normalizedEntryPath || !manifestCache.has(normalizedEntryPath)) {
            if (warnOnMissingEntry) {
              logger$1.warn(`No matching entry point for: ${outputPath}`);
            }
            stats.skipped++;
            continue;
          }
          try {
            const manifest = manifestCache.get(normalizedEntryPath) || {};
            const banner = generateScriptableBanner.generateScriptableBanner(manifest);
            if (result.outputFiles) {
              const outputFile = result.outputFiles.find(
                (file) => utils.isSamePath(file.path, outputPath) || file.path === "<stdout>"
              );
              if (outputFile) {
                outputFile.contents = Buffer.from(banner + outputFile.text.trimEnd());
                stats.success++;
                logger$1.success(`[${progress}%] Added banner to ${outputPath}`);
              }
            } else {
              try {
                await utils.waitForFile(outputPath);
                const content = await promises.readFile(outputPath, "utf-8");
                const updatedContent = banner + content.trimEnd();
                await promises.writeFile(outputPath, updatedContent, "utf8");
                stats.success++;
                logger$1.success(`[${progress}%] Added banner to ${outputPath}`);
              } catch (error) {
                stats.failed++;
                logger$1.error(`[${progress}%] Failed to process ${outputPath}`);
              }
            }
          } catch (error) {
            stats.failed++;
            logger$1.error(`[${progress}%] Failed to process ${outputPath}`);
          }
        }
        logger$1.timeEnd("Banner Processing");
        logger$1.report(stats);
      });
    }
  };
}

exports.scriptableBanner = scriptableBanner;
//# sourceMappingURL=scriptable-banner.js.map
//# sourceMappingURL=scriptable-banner.js.map