import chalk from 'chalk';

function convertEsbuildLogLevel(level) {
  switch (level) {
    case "debug":
      return "verbose";
    case "info":
      return "info";
    case "warning":
      return "warn";
    case "error":
      return "error";
    case "silent":
      return "silent";
    default:
      return "info";
  }
}
class ConsoleLogger {
  constructor(prefix, level = "info") {
    this.prefix = prefix;
    this.level = level;
  }
  timers = /* @__PURE__ */ new Map();
  LOG_LEVELS = {
    silent: 0,
    error: 1,
    warn: 2,
    info: 3,
    verbose: 4
  };
  formatPrefix() {
    return chalk.cyan(`[${this.prefix}]`);
  }
  formatTime(ms) {
    return chalk.gray(`(${ms}ms)`);
  }
  shouldLog(messageLevel) {
    if (this.level === "silent") {
      return false;
    }
    return this.LOG_LEVELS[messageLevel] <= this.LOG_LEVELS[this.level];
  }
  info(message, ...args) {
    if (this.shouldLog("info")) {
      console.log(`${this.formatPrefix()} ${message}`, ...args);
    }
  }
  warn(message, ...args) {
    if (this.shouldLog("warn")) {
      console.warn(`${this.formatPrefix()} ${chalk.yellow("warning")} ${message}`, ...args);
    }
  }
  error(message, ...args) {
    if (this.shouldLog("error")) {
      console.error(`${this.formatPrefix()} ${chalk.red("error")} ${message}`, ...args);
    }
  }
  success(message, ...args) {
    if (this.shouldLog("info")) {
      console.log(`${this.formatPrefix()} ${chalk.green("\u2713")} ${message}`, ...args);
    }
  }
  verbose(message, ...args) {
    if (this.shouldLog("verbose")) {
      console.log(`${this.formatPrefix()} ${chalk.gray(message)}`, ...args);
    }
  }
  time(label) {
    if (this.shouldLog("verbose")) {
      this.timers.set(label, performance.now());
    }
  }
  timeEnd(label) {
    if (this.shouldLog("verbose")) {
      const start = this.timers.get(label);
      if (start) {
        const duration = Math.round(performance.now() - start);
        console.log(`${this.formatPrefix()} ${label} ${this.formatTime(duration)}`);
        this.timers.delete(label);
      }
    }
  }
  report(stats) {
    if (this.level === "silent") return;
    const { total, success, skipped, failed } = stats;
    const summary = [
      `Total: ${total}`,
      `\u2713 Success: ${chalk.green(success)}`,
      skipped > 0 ? `\u26A0\uFE0F Skipped: ${chalk.yellow(skipped)}` : null,
      failed > 0 ? `\u274C Failed: ${chalk.red(failed)}` : null
    ].filter(Boolean).join("  ");
    console.log(`${this.formatPrefix()} Build Result: ${summary}`);
  }
}
function createLogger(prefix, options = {}) {
  return new ConsoleLogger(prefix, options.level ?? "info");
}

export { convertEsbuildLogLevel, createLogger };
//# sourceMappingURL=logger.js.map
//# sourceMappingURL=logger.js.map