import { generateScriptableBanner } from "@scriptables/manifest/generateScriptableBanner";
import { normalizeManifest } from "@scriptables/manifest/normalizeManifest";
import { validateManifest } from "@scriptables/manifest/validateManifest";
import { existsSync } from "fs";
import { readFile, writeFile } from "fs/promises";
import { dirname, basename, resolve } from "path";
import { convertEsbuildLogLevel, createLogger } from "./logger.js";
import { isMatch, normalizeAndResolvePath, isSamePath, waitForFile } from "./utils.js";
const DEFAULT_MANIFEST_EXTENSIONS = [".manifest.json", ".manifest", ".json"];
function scriptableBanner(options = {}) {
    const { logLevel = "auto", manifestExtensions = DEFAULT_MANIFEST_EXTENSIONS, warnOnMissingManifest = true, warnOnInvalidManifest = true, warnOnMissingEntry = true, allowExtraManifestKeys = true, getManifestPath, patterns = ["*.{ts,tsx,js,jsx}"] } = options;
    return {
        name: "scriptable-banner",
        setup(build) {
            const level = logLevel === "auto" ? convertEsbuildLogLevel(build.initialOptions.logLevel || "info") : logLevel;
            const logger = createLogger("scriptable-banner", { level });
            let manifestCache = /* @__PURE__ */ new Map();
            if (!build.initialOptions.metafile) {
                logger.warn("Warning: \"metafile\" option was not enabled. Automatically setting \"metafile: true\" to allow scriptable plugin to modify output files.");
                build.initialOptions.metafile = true;
            }
            build.onStart(() => {
                manifestCache.clear();
            });
            build.onResolve({ filter: /.*/ }, async (args) => {
                if (args.kind !== "entry-point" || !isMatch(args.path, patterns)) {
                    return null;
                }
                const normalizedPath = normalizeAndResolvePath(args.path);
                if (manifestCache.has(normalizedPath)) {
                    return null;
                }
                const dir = dirname(args.path);
                const baseFileName = basename(args.path).replace(/\.[tj]sx?$/, "");
                const manifestPath = getManifestPath ? getManifestPath(args.path) : manifestExtensions.map((ext) => resolve(dir, `${baseFileName}${ext}`)).find(existsSync);
                if (manifestPath) {
                    try {
                        const manifestContent = await readFile(manifestPath, "utf8");
                        const manifest = normalizeManifest(JSON.parse(manifestContent));
                        try {
                            validateManifest(manifest, allowExtraManifestKeys);
                            manifestCache.set(normalizedPath, manifest);
                        }
                        catch (error) {
                            if (warnOnInvalidManifest && error instanceof Error) {
                                logger.warn(`Invalid manifest file: ${manifestPath} - ${error.message}`, error);
                            }
                        }
                    }
                    catch (error) {
                        logger.warn(`Failed to read manifest file: ${manifestPath}`, error);
                    }
                }
                else if (warnOnMissingManifest) {
                    logger.warn(`No manifest file found for entry point: ${args.path}`);
                }
                return null;
            });
            build.onEnd(async (result) => {
                logger.time("Banner Processing");
                if (!result.metafile || Object.keys(result.metafile.outputs).length === 0) {
                    logger.warn("Skipping banner addition because \"metafile\" is not available or contains no outputs. Ensure \"metafile: true\" is set in esbuild options if write is enabled.");
                    return;
                }
                const stats = {
                    total: 0,
                    success: 0,
                    skipped: 0,
                    failed: 0
                };
                const outputs = Object.keys(result.metafile.outputs);
                stats.total = outputs.length;
                logger.info(`Processing ${stats.total} files...`);
                for (const [index, outputPath] of outputs.entries()) {
                    const progress = Math.round((index + 1) / stats.total * 100);
                    logger.verbose(`[${progress}%] Processing: ${outputPath}`);
                    const outputMeta = result.metafile.outputs[outputPath];
                    const entryPoint = outputMeta?.entryPoint ?? "";
                    const normalizedEntryPath = entryPoint ? normalizeAndResolvePath(entryPoint) : "";
                    if (!normalizedEntryPath || !manifestCache.has(normalizedEntryPath)) {
                        if (warnOnMissingEntry) {
                            logger.warn(`No matching entry point for: ${outputPath}`);
                        }
                        stats.skipped++;
                        continue;
                    }
                    try {
                        const manifest = manifestCache.get(normalizedEntryPath) || {};
                        const banner = generateScriptableBanner(manifest);
                        if (result.outputFiles) {
                            const outputFile = result.outputFiles.find((file) => isSamePath(file.path, outputPath) || file.path === "<stdout>");
                            if (outputFile) {
                                outputFile.contents = Buffer.from(banner + outputFile.text.trimEnd());
                                stats.success++;
                                logger.success(`[${progress}%] Added banner to ${outputPath}`);
                            }
                        }
                        else {
                            try {
                                await waitForFile(outputPath);
                                const content = await readFile(outputPath, "utf-8");
                                const updatedContent = banner + content.trimEnd();
                                await writeFile(outputPath, updatedContent, "utf8");
                                stats.success++;
                                logger.success(`[${progress}%] Added banner to ${outputPath}`);
                            }
                            catch (error) {
                                stats.failed++;
                                logger.error(`[${progress}%] Failed to process ${outputPath}`);
                            }
                        }
                    }
                    catch (error) {
                        stats.failed++;
                        logger.error(`[${progress}%] Failed to process ${outputPath}`);
                    }
                }
                logger.timeEnd("Banner Processing");
                logger.report(stats);
            });
        }
    };
}
export { scriptableBanner };
//# sourceMappingURL=scriptable-banner.js.map
//# sourceMappingURL=scriptable-banner.js.map
